@@ -1,8 +1,9 @@
 import IBLPy.config as cfg
-import IBLPy.api_brain as api_brain
-from IBLPy.base_fn import async_api, sync_api, bot_only, InvalidMode, IBLAPIResponse
-from IBLPy import api_modes, fastapi, uvicorn, ws, autoposter, discord
-from typing import Optional
+import IBLPy.http as _http
+from IBLPy.classes import InvalidMode, IBLAPIResponse
+from IBLPy import api_modes, fastapi, uvicorn, ws, discord
+from loguru import logger
+from typing import Optional, Awaitable
 import os
 import asyncio
 import time
@@ -13,20 +14,19 @@ class BotClient():
             
         :param id: The Bot ID you wish to use with the IBL API
 
-        :param api_token: The API Token of the bot. You can find this by clicking API Token under the "Owner Section". This is optional however you will not be able to post stats if you do not pass a API Token
-
-        :param error_on_ratelimit: A boolean to indicate whether we should error with a IBLAPIRatelimit or simply return a IBLAPIResponse on a ratelimit error. Defaults to True
-
-        :param debug: Whether to run in debug mode or not. Defaults to False.
+        :param api_token: The API Token of the bot. You can find this by clicking API Token under the "Owner Section". 
+            This is optional however you will not be able to post stats if you do not pass a API Token
     """
 
-    def __init__(self, id: int, api_token: Optional[str] = "", error_on_ratelimit: bool = True, debug: bool = False):
-        self.id = id
-        self.api_token = api_token
-        self.error_on_ratelimit = error_on_ratelimit
-        self.debug = debug
+    def __init__(self, bot_id: int, api_token: Optional[str] = ""):
+        self.bot_id = bot_id
+        self.http = _http.BotHTTP(bot_id)
+        
+        if api_token:
+            self.http.login(api_token)
 
-    async def set_stats_async(self, guild_count: int, shard_count: Optional[int] = None) -> IBLAPIResponse:
+            
+    async def set_stats(self, guild_count: int, shard_count: Optional[int] = None) -> IBLAPIResponse:
         """
             Posts bot stats to the IBL API asynchronously
             
@@ -37,40 +37,10 @@ class BotClient():
             :return: This will always be returned unless something goes wrong, in which case you will get an exception
             :rtype: IBLAPIResponse 
         """
-        async_api()
-        return await api_brain.set_stats_async(bot_id = self.id, api_token = self.api_token, guild_count = guild_count, shard_count = shard_count, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
+        return await self.http.set_stats(guild_count, shard_count)
+  
     
-    def set_stats(self, guild_count: int, shard_count: Optional[int] = None) -> IBLAPIResponse:
-        """
-            Posts bot stats to the IBL API synchronously
-
-            :param guild_count: Amount of servers your bot is in
-
-            :param shard_count: Amount of shards your bot is in. This is optional
-
-            :return: This will always be returned unless something goes wrong, in which case you will get an exception
-            :rtype: IBLAPIResponse
-        """
-        sync_api()
-        return api_brain.set_stats_sync(bot_id = self.id, api_token = self.api_token, guild_count = guild_count, shard_count = shard_count, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
-    
-    def get_bot(self):
-        """
-            Synchronously get a bot. This does not take any parameters.
-
-            :return: If the bot was not found or if IBLPy could not parse the JSON request for any reason
-            :rtype: None
-
-            :return: This will be returned if you are ratelimited
-            :rtype: IBLAPIResponse
-
-            :return: This is the bot object returned from the API
-            :rtype: IBLBot
-        """
-        sync_api()
-        return api_brain.get_bot_sync(bot_id = self.id, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
-
-    async def get_bot_async(self):
+    async def get_bot(self):
         """
             Asynchronously get a bot. This does not take any parameters.
 
@@ -83,44 +53,21 @@ class BotClient():
             :return: This is the bot object returned from the API
             :rtype: IBLBot
         """
-        async_api()
-        return await api_brain.get_bot_async(bot_id = self.id, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
+        return await self.http.get_bot()
 
 class UserClient():
     """
         Initialize a Infinity Bots List (IBL) User Client. You can use this to get user stats!
             
-        :param id: The User ID you wish to use with the IBL API
-
-        :param error_on_ratelimit: A boolean to indicate whether we should error with a IBLAPIRatelimit or simply return a IBLAPIResponse on a ratelimit error. Defaults to True
-    
-        :param debug: Whether to run in debug mode. Defaults to False.
+        :param user_id: The User ID you wish to use with the IBL API
     """
-    def __init__(self, id: int, error_on_ratelimit: bool = True, debug: bool = False):
-        self.id = id
-        self.error_on_ratelimit = error_on_ratelimit
-        self.debug = debug
+    def __init__(self, user_id: int):
+        self.user_id = user_id
+        self.http = _http.UserHTTP(user_id)
 
-    def get_user(self):
+    async def get_user(self):
         """
-            Synchronously get a user. This does not take any parameters.
-
-            :return: If the user was not found or if IBLPy could not parse the JSON request for any reason
-            :rtype: None
-
-            :return: This will be returned if you are ratelimited
-            :rtype: IBLAPIResponse
-
-            :return: This is the user object returned from the API
-            :rtype: IBLUser
-
-        """
-        sync_api()
-        return api_brain.get_user_sync(user_id = self.id, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
-
-    async def get_user_async(self):
-        """
-            Asynchronously get a user. This does not take any parameters.
+            Get a user. This does not take any parameters.
 
             :return: If the user was not found or if IBLPy could not parse the JSON request for any reason
             :rtype: None
@@ -131,30 +78,58 @@ class UserClient():
             :return: This is the user object returned from the API
             :rtype: IBLUser
         """
-        async_api()
-        return await api_brain.get_user_async(user_id = self.id, error_on_ratelimit = self.error_on_ratelimit, debug = self.debug)
+        return await self.http.get_user()
 
+
+async def _default_get_counts(ap):
+    """Returns guild and shard count for default autoposter"""
+    return {
+        "guild_count": len(ap.discli.guilds),
+        "shard_count": len(ap.discli.shards) if ap.sharding else None
+    }
+    
 class AutoPoster():
     """
-        This is IBL Auto Poster. It will post stats for you on an interval you decide and it will then run a function of your choice (if requested) after posting each time.
+        This is IBL Auto Poster. It will post stats for you on an interval you decide
+            and it will then run a function of your choice (if requested) after posting each time.
 
         You can stop a Auto Poster by calling the ``stop`` function.
+        
+        :param get_counts: A async function that takes in the AutoPoster object and then
+            returns the dict {"guild_count: guild count, "shard_count": shard_count} for the autoposter.
+            A default value for this is provided for ease of use
 
-        :param interval: How long to wait each time you post. It is required to use at least 5 minutes which is 300 seconds. Specifying a value below 300 seconds will set the interval to 300 seconds anyways
+        :param interval: How long to wait each time you post. It is required to use at least 5 minutes which is 300 seconds.
+            Specifying a value below 300 seconds will set the interval to 300 seconds anyways
 
-        :param botcli: The IBL BotClient of the bot you want to post stats for. It must have the API token set either during initialization or afterwards by setting the api_token parameter
+        :param botcli: The IBL BotClient of the bot you want to post stats for. 
+            It must have the API token set either during initialization or afterwards by setting the api_token parameter
 
-        :param discli: The discord.Client (or like interfaces like discord.Bot/discord.AutoShardedBot) of the bot you want to post stats for. If you wish to add shard counts, please give a discord.AutoShardedClient and pass sharding = True
+        :param discli: The discord.Client (or like interfaces like discord.Bot/discord.AutoShardedBot) of the bot you want to post stats for.
+            If you wish to add shard counts, please give a discord.AutoShardedClient and pass sharding = True or use get_counts
 
-        :param on_post: The function to call after posting. Set to None if you don't want this. This function needs to accept three arguments, the guild count sent, the shard count set and the response (which will either be a IBLAPIResponse, a IBLAPIRatelimit or None). Shard count will be None if sharding is disabled
+        :param on_post: The function to call after posting. Set to None if you don't want this.
+            This function needs to accept three arguments, the guild count sent, the shard count set 
+            and the response (which will either be a IBLAPIResponse, a IBLAPIRatelimit or None). 
+            Shard count will be None if sharding is disabled and/or get_count provided doesn't support it
 
         :param sharding: Whether we should post sharding as well. Requires a discord.AutoShardedClient
     """
-    def __init__(self, interval: int, botcli: BotClient, discli: discord.Client, sharding = None, on_post = None):
+    def __init__(
+        self, 
+        interval: int,
+        botcli: BotClient, 
+        discli: discord.Client, 
+        sharding = None, 
+        on_post = None,
+        get_counts: Awaitable = _default_get_counts,
+    ):
+
         if interval > 300:
             self.interval = interval
         else:
             self.interval = 300
+        self.get_counts = get_counts
         self.botcli = botcli
         self.discli = discli
         self.on_post = on_post
@@ -167,25 +142,26 @@ class AutoPoster():
         while True:
             if not self.keep_running:
                 self.keep_running = True
-                print("IBLPy: AutoPoster has exited successfully!")
+                logger.info("IBLPy: AutoPoster has exited successfully!")
                 return
 
             if self.interval < 300:
                 self.interval = 300
 
             try:
-                if self.sharding in [False, None]:
-                    res = await self.botcli.set_stats_async(len(self.discli.guilds))
-                else:
-                    res = await self.botcli.set_stats_async(len(self.discli.guilds), len(self.discli.shards))
+                
+                try:
+                    counts = await self.get_counts(self)
+                except Exception as exc:
+                    logger.warning("Using self in get_counts did not work, trying no args...")
+                    counts = await self.get_counts()
+                    
+                res = await self.botcli.set_stats(counts["guild_count"], counts["shard_count"])
                 if self.on_post:
-                    if self.sharding in [False, None]:
-                        await self.on_post(len(self.discli.guilds), None, res)
-                    else:
-                        await self.on_post(len(self.discli.guilds), len(self.discli.shards), res)
-                print(f"IBLPy: Stats posted successfully at {time.time()}")
+                    await self.on_post(counts["guild_count"], counts["shard_count"], res)
+                logger.success(f"IBLPy: Stats posted successfully at {time.time()}")
             except Exception as ex:
-                print(f"IBLPy: Could not post stats because: {ex}")
+                logger.error(f"IBLPy: Could not post stats because: {ex}")
             await asyncio.sleep(self.interval)
 
     def start(self):
